@@ -1,11 +1,15 @@
+import time
+
 import gtk
 
-from snaked.core.shortcuts import ContextShortcutActivator
+from snaked.core.shortcuts import ContextShortcutActivator, register_shortcut
 import snaked.core.editor
 
 class TabbedEditorManager(snaked.core.editor.EditorManager):
-    def __init__(self):
+    def __init__(self, show_tabs=True):
         super(TabbedEditorManager, self).__init__()
+        
+        self.last_switch_time = None
 
         self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
         self.window.connect('delete-event', self.on_delete_event)
@@ -16,10 +20,20 @@ class TabbedEditorManager(snaked.core.editor.EditorManager):
         self.activator = ContextShortcutActivator(self.window, self.get_context)
         
         self.note = gtk.Notebook()
+        self.note.set_show_tabs(show_tabs)
+        self.note.set_scrollable(True)
         self.note.set_property('tab-hborder', 10)
         self.note.set_property('homogeneous', False)
         self.note.connect_after('switch-page', self.on_switch_page)
+        self.note.connect('page_removed', self.on_page_removed)
         self.window.add(self.note)
+
+        register_shortcut('toggle-tabs-visibility', '<alt>F11', 'Window', 'Toggles tabs visibility')
+        register_shortcut('next-editor', '<alt>Right', 'Window', 'Switches to next editor')
+        register_shortcut('prev-editor', '<alt>Left', 'Window', 'Switches to previous editor')
+        register_shortcut('next-editor-alt', '<ctrl>Page_Down', 'Window', 'Switches to next editor')
+        register_shortcut('prev-editor-alt', '<ctrl>Page_Up', 'Window', 'Switches to previous editor')
+        register_shortcut('fullscreen', 'F11', 'Window', 'Toggles fullscreen mode')
         
         self.window.show_all()
     
@@ -33,7 +47,7 @@ class TabbedEditorManager(snaked.core.editor.EditorManager):
 
     def manage_editor(self, editor):
         label = gtk.Label('Unknown')
-        self.note.append_page(editor.widget, label)
+        self.note.insert_page(editor.widget, label, -1)
         self.focus_editor(editor)
         editor.view.grab_focus()
        
@@ -76,9 +90,17 @@ class TabbedEditorManager(snaked.core.editor.EditorManager):
         self.activator.bind_to_name('save', self.save)
         self.activator.bind_to_name('next-editor', self.switch_to, 1)
         self.activator.bind_to_name('prev-editor', self.switch_to, -1)
+        self.activator.bind_to_name('next-editor-alt', self.switch_to, 1)
+        self.activator.bind_to_name('prev-editor-alt', self.switch_to, -1)
         self.activator.bind_to_name('new-file', self.new_file_action)
         self.activator.bind_to_name('show-preferences', self.show_preferences)
         self.activator.bind_to_name('fullscreen', self.fullscreen, [True])
+        self.activator.bind_to_name('toggle-tabs-visibility', self.toggle_tabs)
+
+        self.activator.bind_to_name('place-spot', self.add_spot_with_feedback)
+        self.activator.bind_to_name('goto-last-spot', self.goto_last_spot)
+        self.activator.bind_to_name('goto-next-spot', self.goto_next_prev_spot, True)
+        self.activator.bind_to_name('goto-prev-spot', self.goto_next_prev_spot, False)
 
         self.activator.bind('Escape', self.process_escape)
 
@@ -94,8 +116,24 @@ class TabbedEditorManager(snaked.core.editor.EditorManager):
 
     def on_switch_page(self, *args):
         self.update_top_level_title()
+
+    def on_page_removed(self, note, child, idx):
+        for e in self.editors:
+            if e.widget is child:
+                spot = self.get_last_spot(None, e)
+                if spot:
+                    note.set_current_page(note.page_num(spot.editor().widget))
+                    return
+
+        if idx > 0:
+            note.set_current_page(idx - 1)
         
     def switch_to(self, editor, dir):
+        if self.last_switch_time is None or time.time() - self.last_switch_time > 5:
+            self.add_spot(editor)
+            
+        self.last_switch_time = time.time()
+        
         idx = ( self.note.get_current_page() + dir ) % self.note.get_n_pages()
         self.note.set_current_page(idx)
 
@@ -105,4 +143,7 @@ class TabbedEditorManager(snaked.core.editor.EditorManager):
         else:
             self.window.unfullscreen()
             
-        state[0] = not state[0]
\ No newline at end of file
+        state[0] = not state[0]
+        
+    def toggle_tabs(self, editor):
+        self.note.set_show_tabs(not self.note.get_show_tabs())
\ No newline at end of file
