@@ -1,12 +1,54 @@
 import os
 
+import weakref
 import gtk
 import gio
 
-from snaked.signals import connect_external, connect_all
-from snaked.util import idle, refresh_gui, lazy_property
+from snaked.signals import connect_external, connect_all, weak_connect
+from snaked.util import idle, lazy_property
 
-from .ropehints import FileHintDb
+from .ropehints import CompositeHintProvider
+
+project_managers = weakref.WeakValueDictionary()
+
+class RopeProjectManager(object):
+    def __init__(self, project):
+        self.project = project
+        self.hints_monitor = None
+
+        if project.ropefolder:
+            self.hints_filename = os.path.join(project.ropefolder.real_path, 'ropehints.py')        
+            self.hints_monitor = gio.File(self.hints_filename).monitor_file()
+            weak_connect(self.hints_monitor, 'changed', self, 'on_hints_file_changed')
+
+            self.refresh_hints()            
+
+    def refresh_hints(self):
+        self.project.pycore.module_cache.forget_all_data()
+        self.project.pycore.hintdb = CompositeHintProvider(self.project)
+
+        if self.hints_filename and os.path.exists(self.hints_filename):
+            namespace = {}
+            execfile(self.hints_filename, namespace)
+            if 'init' in namespace:
+                try:
+                    namespace['init'](self.project.pycore.hintdb)
+                except:
+                    import traceback
+                    traceback.print_exc()
+            
+    def on_hints_file_changed(self, filemonitor, file, other_file, event):
+        if event in (gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT, gio.FILE_MONITOR_EVENT_CREATED):        
+            self.refresh_hints()
+            
+    def close(self):
+        if self.hints_monitor:
+            self.hints_monitor.cancel()
+            self.hints_monitor = None
+        self.project.close()
+        
+    def __del__(self):
+        self.close()
 
 class Plugin(object):
     def __init__(self, editor):
@@ -16,36 +58,30 @@ class Plugin(object):
 
     def init_completion(self):
         provider = self.completion_provider
-        self.editor.view.get_completion().add_provider(provider)
+        completion = self.editor.view.get_completion()
+        completion.add_provider(provider)
+        #completion.get_info_window().set_sizing(300, 400, False, False)
         
-    def close(self):
-        if hasattr(self, '__project'):
-            self.project.close()
-            self.hints_monitor.cancel()
-    
-    def refresh_hints(self, filemonitor, file, other_file, event, db, project):
-        if event in (gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT, gio.FILE_MONITOR_EVENT_CREATED):        
-            db.refresh()
-            project.pycore.module_cache.forget_all_data()
-
     @lazy_property
-    def project(self):
-        if not os.access(self.editor.uri, os.W_OK):
-            root = '/tmp'
-        else:
-            root = self.editor.project_root
-        
-        if root:
-            from rope.base.project import Project
-            project = Project(root)
-            db = FileHintDb(project)
+    def project_manager(self):
+        root = getattr(self.editor, 'ropeproject_root', self.editor.project_root)
+        try:
+            return project_managers[root]
+        except KeyError:
+            pass
 
-            self.hints_monitor = gio.File(db.hints_filename).monitor_file()
-            self.hints_monitor.connect('changed', self.refresh_hints, db, project)
-            
-            return project
+        if os.access(root, os.W_OK):
+            kwargs = {}
         else:
-            return None           
+            kwargs = dict(ropefolder=None)
+        
+        from rope.base.project import Project
+        project = Project(root, **kwargs)
+        project.snaked_project_root = root
+        
+        pm = RopeProjectManager(project)
+        project_managers[root] = pm
+        return pm
     
     @lazy_property
     def completion_provider(self):
@@ -53,10 +89,13 @@ class Plugin(object):
         return complete.RopeCompletionProvider(self)
 
     def get_rope_resource(self, project, uri=None):
-        from rope.base import libutils    
+        from rope.base import libutils, exceptions    
         uri = uri or self.editor.uri
-        return libutils.path_to_resource(project, uri)
-
+        try:
+            return libutils.path_to_resource(project, uri)
+        except exceptions.ResourceNotFoundError:
+            return None
+            
     def get_source_and_offset(self):
         offset = self.editor.cursor.get_offset()
         source = self.editor.text
@@ -66,14 +105,7 @@ class Plugin(object):
         
         return source, offset
 
-    def goto_line(self, editor, line):
-        refresh_gui()
-        edit = editor.buffer
-        iterator = edit.get_iter_at_line(line - 1)
-        edit.place_cursor(iterator)
-        editor.view.scroll_to_iter(iterator, 0.001, use_align=True, xalign=1.0)
-    
-    def get_fuzzy_location(self, source, offset):
+    def get_fuzzy_location(self, project, source, offset):
         from rope.base import worder, exceptions
         
         word_finder = worder.Worder(source, True)
@@ -83,7 +115,7 @@ class Plugin(object):
         names = expression.split('.')
         pyname = None
         try:
-            obj = self.project.pycore.get_module(names[0])
+            obj = project.pycore.get_module(names[0])
             for n in names[1:]:
                 pyname = obj[n]
                 obj = pyname.get_object()
@@ -108,13 +140,8 @@ class Plugin(object):
         return resource, line
 
     def goto_definition(self):
-        project = self.project
-        if not project:
-            project = getattr(self.editor, 'ropeproject', None)
-            if not project:
-                self.editor.message("Can't find project path")
-                return
-        
+        project = self.project_manager.project
+
         project.validate()
 
         current_resource = self.get_rope_resource(project) 
@@ -133,22 +160,22 @@ class Plugin(object):
             return
 
         if resource is None and line is None:
-            resource, line = self.get_fuzzy_location(source, offset)
+            resource, line = self.get_fuzzy_location(project, source, offset)
         
         if resource and resource.real_path == current_resource.real_path:
             resource = None
             
         if resource:
             uri = resource.real_path
-            editor = self.editor.open_file(uri, line-1)
-            editor.ropeproject = project 
+            editor = self.editor.open_file(uri, line - 1)
+            editor.ropeproject_root = project.snaked_project_root 
         else:
             if line:
-                self.goto_line(self.editor, line)
+                self.editor.add_spot()
+                self.editor.goto_line(line)
             else:
                 self.editor.message("Unknown definition")
 
-
     @connect_external('view', 'key-press-event')
     def on_textview_key_press_event(self, sender, event):
         if event.keyval != gtk.keysyms.Return:
@@ -215,13 +242,7 @@ class Plugin(object):
         return True
 
     def show_calltips(self):
-        project = self.project
-        if not project:
-            project = getattr(self.editor, 'ropeproject', None)
-            if not project:
-                self.editor.message("Can't find project path")
-                return
-        
+        project = self.project_manager.project
         project.validate()
 
         current_resource = self.get_rope_resource(project) 
